fieldset {
    border: 1px groove #ddd !important;
    padding: 0 1.4em 1.4em 1.4em !important;
    margin: 0 0 1.5em 0 !important;
    -webkit-box-shadow:  0px 0px 0px 0px #000;
            box-shadow:  0px 0px 0px 0px #000;
}

legend {
    text-align: left !important;
    width:inherit;
    padding:0 10px;
    border-bottom:none;
    margin-bottom:0;
}

.monospace-style {
    font-family: monospace;
}

.help-icon {
    cursor: pointer;
}

.div-fake-hidden {
    width:0px;
    height:0px;
    overflow:hidden;
}

.table-borderless tbody tr td,
.table-borderless tbody tr th,
.table-borderless thead tr th,
.table-borderless thead tr td,
.table-borderless tfoot tr th,
.table-borderless tfoot tr td {
    border: none;
}

/* SVG axis (Has been intergrated in primer.js)
.axis path,
.axis line{
    fill: none;
    stroke: blue;
    stroke-width: 2px;
    shape-rendering: crispEdges;
}

.axis text {
    font-size: 11px;
}
.primerLabel {
    font-size: 11px;
}
.primerUniqueLabel {
    font-size: 11px;
    font-weight: bold;
}*/

/* Sticky footer styles
-------------------------------------------------- */
html {
  position: relative;
  min-height: 100%;
}
body {
  /* Margin bottom by footer height */
  margin-bottom: 80px;
}
.footer {
  position: absolute;
  bottom: 0;
  width: 100%;
  /* Set the fixed height of the footer here */
  height: 60px;
  background-color: #D9DBE0;
  text-align: center;
}
.container .text-muted {
  margin: 20px 0;
}